@@ -80,44 +80,60 @@ def get_routes(config):
 @workers.register(pool_size=1)
 def aggregate_data_worker(app):
     # Worker in charge of aggregate data
-    try:
-        engine = worker_engine(app.config.repository)
-        with engine.begin() as conn:
-            conn.execute("SET search_path TO monitoring")
-            logger.debug("Running SQL function monitoring.aggregate_data()")
-            res = conn.execute("SELECT * FROM aggregate_data()")
-            for row in res.fetchall():
-                logger.debug("table=%s insert=%s"
-                             % (row['tblname'], row['nb_rows']))
-            return
-    except Exception as e:
-        logger.error('Could not aggregate montitoring data')
-        logger.exception(e)
-        raise(e)
+    engine = worker_engine(app.config.repository)
+    with engine.connect() as conn:
+        conn.execute("SET search_path TO monitoring")
+        res = conn.execute("SELECT * FROM metric_tables_config()")
+        tables_config, = res.fetchone()
+        for config in tables_config.values():
+            logger.info("Aggregating data for metric %s.", config['name'])
+            try:
+                with conn.begin():
+                    res = conn.execute(
+                        "SELECT * FROM aggregate_data_single(%s, %s, %s)", (
+                            config['name'], config['record_type'],
+                            config['aggregate'],
+                        )
+                    )
+                    table_name, nb_rows = res.fetchone()
+                    logger.debug("table=%s insert=%s", table_name, nb_rows)
+            except Exception as e:
+                logger.error("Failed to archive data: %s.", e)
+
+    logger.info("Monitoring data aggregation done.")
 
 
 @workers.register(pool_size=1)
 def history_tables_worker(app):
-    # Worker in charge of history tables
-    try:
-        engine = worker_engine(app.config.repository)
-        with engine.connect() as conn:
-            conn.execute("SET search_path TO monitoring")
-            logger.debug("Running SQL function monitoring.history_tables()")
-            res = conn.execute("SELECT * FROM history_tables()")
-            for row in res.fetchall():
-                logger.debug("table=%s insert=%s"
-                             % (row['tblname'], row['nb_rows']))
-            conn.execute("COMMIT")
-            return
-    except Exception as e:
-        logger.error('Could not history montitoring tables')
-        logger.exception(e)
-        try:
-            conn.execute("ROLLBACK")
-        except Exception:
-            pass
-        raise(e)
+    # Archive monitoring metric tables.
+    #
+    # Copy contents of every metric_*_current tables into corresponding
+    # metric_*_history, aggregated. Then truncate _current tables.
+    #
+    # This task is triggered every 3 hours by monitoring_boostrap() below.
+    #
+    engine = worker_engine(app.config.repository)
+    with engine.connect() as conn:
+        conn.execute("SET search_path TO monitoring")
+        res = conn.execute("SELECT * FROM metric_tables_config()")
+        tables_config, = res.fetchone()
+        for config in tables_config.values():
+            logger.info("Archiving data for metric %s.", config['name'])
+            try:
+                with conn.begin():
+                    res = conn.execute(
+                        "SELECT * FROM archive_current_metrics(%s, %s, %s)", (
+                            config['name'], config['record_type'],
+                            config['history'],
+                        )
+                    )
+
+                    table_name, nb_rows = res.fetchone()
+                    logger.debug("table=%s insert=%s", table_name, nb_rows)
+            except Exception as e:
+                logger.error("Failed to archive data: %s.", e)
+
+    logger.info("Monitoring data archiving done.")
 
 
 @workers.register(pool_size=10)
@@ -318,7 +334,7 @@ def schedule_collector(app):
     """Worker function in charge of scheduling collector (pull mode)."""
 
     logger.setLevel(app.config.logging.level)
-    logger.info("Starting collector scheduler worker.")
+    logger.info("Starting collector scheduler.")
 
     engine = worker_engine(app.config.repository)
     with engine.connect() as conn:
@@ -335,6 +351,8 @@ def schedule_collector(app):
             agent_id = "%s:%s" % (agent_address, agent_port)
             task_id = hashlib.md5(agent_id.encode("utf-8")).hexdigest()[:8]
 
+            logger.info("Scheduling collector for agent %s.", agent_id)
+
             taskmanager.schedule_task(
                 'collector',
                 listener_addr=os.path.join(
@@ -349,19 +367,20 @@ def schedule_collector(app):
                 expire=0,
             )
 
-    logger.info("End of collector scheduler worker.")
+    logger.info("End of collector scheduler.")
 
 
 @workers.register(pool_size=20)
 def collector(app, address, port, key):
-    logger.setLevel(app.config.logging.level)
-    logger.info("Starting collector worker for %s:%s", address, port)
+    agent_id = "%s:%s" % (address, port)
+    logger.info("Starting collector for %s.", agent_id)
 
     discover_data = dict()
 
     # First, we need to call discover API because we want to know the hostname
+    logger.info("Discovering hostname from agent %s.", agent_id)
     url = 'https://%s:%s/discover' % (address, port)
-    logger.debug("Calling %s" % url)
+    logger.debug("Calling %s.", url)
 
     try:
         response = temboard_request(
@@ -371,7 +390,7 @@ def collector(app, address, port, key):
             headers={"Content-type": "application/json"},
         )
     except HTTPError:
-        logger.error("Could not get response from %s" % url)
+        logger.error("Could not get response from %s.", url)
         logger.error("Agent or host are down.")
         return
 
@@ -401,12 +420,15 @@ def collector(app, address, port, key):
             host_id,
             discover_data['pg_port']
         )
+        logger.info(
+            "Found host #%s and instance #%s for agent %s, hostname %s.",
+            host_id, instance_id, agent_id, discover_data['hostname'])
     except Exception:
         # This case happens on the very first pull when no data have been
         # previously added.
         logger.warning(
             "Could not find host or instance records in monitoring inventory "
-            "tables."
+            "tables for agent %s.", agent_id,
         )
     else:
         # Get last inserted data timestamp from collector status
@@ -423,13 +445,15 @@ def collector(app, address, port, key):
     # Finally, let's call /monitoring/history agent API for getting metrics
     # history.
     try:
-        logger.debug("Calling %s" % history_url)
+        logger.info("Querying monitoring history from agent %s.", agent_id)
+        logger.debug("Calling %s.", history_url)
         response = temboard_request(
             app.config.temboard.ssl_ca_cert_file,
             'GET',
             history_url,
             headers={"Content-type": "application/json"},
         )
+        response = json.loads(response)
     except HTTPError as e:
         if e.code == 404:
             logger.error("Agent version does not support pull mode. "
@@ -447,7 +471,10 @@ def collector(app, address, port, key):
                 worker_session.commit()
         return
 
-    for row in json.loads(response):
+    if not response:
+        logger.info("Agent %s returned no monitoring data.", agent_id)
+
+    for row in response:
         hostinfo = row['hostinfo']
         data = row['data']
         instance = row['instances'][0]
@@ -458,17 +485,17 @@ def collector(app, address, port, key):
         try:
             # Try to insert collected data
 
-            # Update the inventory
+            logger.info("Update the inventory for %s.", agent_id)
             host = merge_agent_info(worker_session, hostinfo, instance)
             instance_id = get_instance_id(worker_session, host.host_id, port)
-            # Insert instance availability informations
+            logger.info("Insert instance availability for %s.", agent_id)
             insert_availability(
                 worker_session,
                 row['datetime'],
                 instance_id,
                 row['instances'][0]['available']
             )
-            # Insert collected metrics
+            logger.info("Insert collected metrics for %s.", agent_id)
             insert_metrics(worker_session, host.host_id, instance_id, data)
             worker_session.commit()
 
@@ -489,8 +516,8 @@ def collector(app, address, port, key):
 
             logger.exception(str(e))
             worker_session.rollback()
-            # Set collector status to FAIL
             if instance_id:
+                logger.info("Set collector status to FAIL for %s.", host)
                 update_collector_status(
                     worker_session,
                     instance_id,
@@ -499,10 +526,10 @@ def collector(app, address, port, key):
                     last_insert=last_insert,
                 )
                 worker_session.commit()
-            # Continue with the next row
+            logger.info("Continue with the next row.")
             continue
 
-        # Update collector status
+        logger.debug("Update collector status for agent %s.", agent_id)
         update_collector_status(
             worker_session,
             instance_id,
@@ -514,6 +541,7 @@ def collector(app, address, port, key):
             ),
         )
         worker_session.commit()
+        logger.info("Populate checks for %s.", host)
         # ALERTING PART
         populate_host_checks(
             worker_session,
@@ -523,7 +551,9 @@ def collector(app, address, port, key):
         )
         worker_session.commit()
 
-        # Apply alerting checks against preprocessed data
+        logger.info(
+            "Apply alerting checks against preprocessed data for agent %s.",
+            agent_id)
         check_preprocessed_data(
             worker_session,
             host.host_id,
@@ -539,7 +569,7 @@ def collector(app, address, port, key):
         logger.debug("Row with datetime=%s inserted", row['datetime'])
         worker_session.commit()
 
-    logger.info("End of collector worker.")
+    logger.info("End of collector for agent %s.", agent_id)
 
 
 @taskmanager.bootstrap()
@@ -565,6 +595,7 @@ def monitoring_bootstrap(context):
     yield taskmanager.Task(
             worker_name='schedule_collector',
             id='schedule_collector',
-            redo_interval=60,  # Repeat each 60s
+            # redo_interval=60,  # Repeat each 60s
+            redo_interval=10,  # Repeat each 60s
             options={},
     )
