@@ -1,3 +1,4 @@
+from builtins import str
 from datetime import datetime
 import logging
 
@@ -7,9 +8,9 @@ from temboardui.web import (
     csvify,
 )
 
+from .. import PLUGIN_NAME
 from . import blueprint, render_template
 from ..chartdata import (
-    get_availability,
     get_unavailability_csv,
     get_metric_data_csv,
 )
@@ -28,7 +29,7 @@ from ..tools import (
 )
 from ..model.db import insert_availability
 
-logger = logging.getLogger('temboardui.plugins.' + __name__)
+logger = logging.getLogger(__name__)
 
 
 def check_agent_request(request, hostname, instance):
@@ -47,18 +48,6 @@ def check_agent_request(request, hostname, instance):
         check_host_key(request.db_session, hostname, key)
 
 
-@blueprint.instance_route("/monitoring/availability")
-def availability(request):
-    request.instance.check_active_plugin('monitoring')
-    try:
-        host_id, instance_id = get_request_ids(request)
-        data = get_availability(request.db_session, host_id, instance_id)
-        return {'available': data}
-    except Exception as e:
-        logger.exception(str(e))
-        return {'available': False}
-
-
 @blueprint.route(r"/(?:monitoring|supervision)/collector",
                  methods=['POST'], json=True)
 @anonymous_allowed
@@ -136,6 +125,7 @@ def collector(request):
 
 @blueprint.instance_route("/monitoring")
 def index(request):
+    request.instance.check_active_plugin(PLUGIN_NAME)
     try:
         agent_username = request.instance.get_profile()['username']
     except Exception:
